@@ -395,6 +395,23 @@ read_config(char *file, struct cfentry *cfe, int count, void *data)
                             *((char **) (data + cfe[i].cf_dataoffset)) =
                                 estr;
                         }
+                        break;
+                    }
+                case CF_FUNCTION:
+                    {
+                        char *estr;
+
+                        estr = (*cfe[i].cf_check)(
+                            str,
+                            (data + cfe[i].cf_dataoffset)
+                        );
+                        if (estr != NULL) {
+                            SYSLOG(LOG_WARNING, ERR_CONF_CHECKFUNC,
+                                   file, nline, estr);
+                            error++;
+                            break;
+                        }
+
                         break;
                     }
                 default:
