@@ -58,6 +58,7 @@ extern int dgconfig_loglevel;
 #define CF_INTEGER      1
 #define CF_INT_PLUS     2
 #define CF_STRING       3
+#define CF_FUNCTION     4
 
 /* LDAPåؿ */
 #define LDAP_VERSION_MIN     2
@@ -87,7 +88,7 @@ if (dgconfig_loglevel >= LOGLVL_ERROR) { \
 }
 
 /* եܾǼ¤Τǻ */
-#define OFFSET(x, y) ((int)&(((x *)NULL)->y))
+#define OFFSET(x, y) ((size_t)&(((x *)NULL)->y))
 
 
 /*--- 顼å ---*/
