@@ -27,6 +27,9 @@
 #ifndef _GLOBAL_H_
 #define _GLOBAL_H_
 
+/* SaMMA version */
+#define SAMMA_VERSION "4.1.4"
+
 /* Return Code */
 #define SUCCESS		0
 #define ERROR		-1
@@ -54,7 +57,9 @@
 #endif
 
 /* Define the run mode flag of SaMMA. */
+extern int ismode_enc;
 extern int ismode_delete;  // 0 : Encryption mode
                            // 1 : Delete mode
+extern int ismode_harmless;
 
 #endif /* _GLOBAL_H_ */
