@@ -18,12 +18,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
-/*
- * $RCSfile: mailzip.c,v $
- * $Revision: 1.22 $
- * $Date: 2011/12/28 03:22:21 $
- */
-
 #include <stdio.h>
 #include <unistd.h>
 #include <errno.h>
@@ -50,11 +44,34 @@
 #include "samma_autobcc.h"
 #include "global.h"
 #include "netlist.h"
+#include "harmless.h"
+#include "sender_check.h"
+
+/* milter struct */
+struct smfiDesc smfilter =
+{
+    IDENT,
+    SMFI_VERSION,   /* version code -- do not change */
+    SMFIF_DELRCPT,  /* flags */
+    mlfi_connect,   /* connection info filter */
+    mlfi_helo,           /* SMTP HELO command filter */
+    mlfi_envfrom,   /* envelope sender filter */
+    mlfi_envrcpt,   /* envelope recipient filter */
+    mlfi_header,    /* header filter */
+    mlfi_eoh,       /* end of header */
+    mlfi_body,      /* body block filter */
+    mlfi_eom,       /* end of message */
+    mlfi_abort,     /* message aborted */
+    mlfi_close      /* connection cleanup */
+};
+
 
 #define MLFIPRIV(ctx)        ((struct mlfiPriv *) smfi_getpriv(ctx))
 
 /* Initialize as Encryption Mode */
+int ismode_enc = 1;
 int ismode_delete = 0;
+int ismode_harmless = 0;
 
 /*
  *  sfsistat mlfi_connect
@@ -76,40 +93,41 @@ mlfi_connect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *addr)
     DEBUGLOG("mlfi_connect start");
 
     /* START ADD: 20150316 */
-    /* if is IPv4 */ 
-    if(addr->sa_family == AF_INET) { 
-        if(inet_ntop(AF_INET, &s->sin_addr, ipv4str, sizeof(ipv4str)) == NULL) {
-            log(ERR_WHITELIST_INVALID_IP, "mlfi_connect", ipv4str); 
-            return SMFIS_ACCEPT;
-        }
+    /* if is IPv4 */
+    if (addr != NULL) {
+        if(addr->sa_family == AF_INET) {
+            if(inet_ntop(AF_INET, &s->sin_addr, ipv4str, sizeof(ipv4str)) == NULL) {
+                log(ERR_WHITELIST_INVALID_IP, "mlfi_connect", ipv4str);
+                return SMFIS_ACCEPT;
+            }
 
-        /* check ip in whitelist file  */
-        check_flg = check_whitelist_file(AF_INET, ipv4str);
+            /* check ip in whitelist file  */
+            check_flg = check_whitelist_file(AF_INET, ipv4str);
 
-        /* file attach do not encrypt */
-        if (check_flg == NOT_ENC) {
-            log(WHITELIST_NOT_ENCRYPTION,"mlfi_connect", ipv4str); 
-            return SMFIS_ACCEPT;
-        }
+            /* file attach do not encrypt */
+            if (check_flg == NOT_ENC) {
+                log(WHITELIST_NOT_ENCRYPTION,"mlfi_connect", ipv4str);
+                return SMFIS_ACCEPT;
+            }
 
-    /* if is IPv6 */
-    } else if (addr->sa_family == AF_INET6) {
-        if (inet_ntop(AF_INET6, &s6->sin6_addr, ipv6str, sizeof(ipv6str)) == NULL) {
-            log(ERR_WHITELIST_INVALID_IP, "mlfi_connect", ipv6str); 
-            return SMFIS_ACCEPT;
-        }
-        /* check ip in whitelist file  */
-        check_flg = check_whitelist_file(AF_INET6, ipv6str);
-        /* file attach do not encrypt */
-        if (check_flg == NOT_ENC) {
-            log(WHITELIST_NOT_ENCRYPTION,"mlfi_connect", ipv6str);
-            return SMFIS_ACCEPT;
+        /* if is IPv6 */
+        } else if (addr->sa_family == AF_INET6) {
+            if (inet_ntop(AF_INET6, &s6->sin6_addr, ipv6str, sizeof(ipv6str)) == NULL) {
+                log(ERR_WHITELIST_INVALID_IP, "mlfi_connect", ipv6str);
+                return SMFIS_ACCEPT;
+            }
+            /* check ip in whitelist file  */
+            check_flg = check_whitelist_file(AF_INET6, ipv6str);
+            /* file attach do not encrypt */
+            if (check_flg == NOT_ENC) {
+                log(WHITELIST_NOT_ENCRYPTION,"mlfi_connect", ipv6str);
+                return SMFIS_ACCEPT;
+            }
         }
 
     /* unknown */
     } else {
-       fprintf(stderr, "%s: socket family %d is not supported\n",
-                                          IDENT, addr->sa_family);
+       log("%s: socket family is not supported\n", IDENT);
     }
 
     /* ENDADD 20150316 */
@@ -117,7 +135,7 @@ mlfi_connect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *addr)
     /* mail save */
     priv = malloc(sizeof *priv);
     if(priv == NULL) {
-	log(ERR_MEMORY_ALLOCATE, "mlfi_connect", "priv", strerror(errno));
+        log(ERR_MEMORY_ALLOCATE, "mlfi_connect", "priv", strerror(errno));
         return SMFIS_TEMPFAIL;
     }
 
@@ -126,50 +144,88 @@ mlfi_connect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *addr)
     /* read config */
     priv->mlfi_conf = config_init();
 
-    /* set RcptTo check flag */
     priv->mlfi_rcptcheck = BEFOR_RCPTCHECK;
 
     /* set encription flag */
-    priv->mlfi_encstatus = ENC; //initialization for Encryption mode and Delete mode.
+    priv->mlfi_encstatus = ENC;
+    //initialization for Encryption mode and Delete mode.
 
-    /* Only for Encription mode --- from here --- */
-    if (!ismode_delete) {
-    
-        /* set AutoBcc check flag */
-        if (priv->mlfi_conf->cf_autobccoption[0] == 'Y' || priv->mlfi_conf->cf_autobccoption[0] == 'y') {
-	    priv->mlfi_bccoption = ENABLE_AUTOBCC;
+    if (ismode_enc) {
+        if (priv->mlfi_conf->cf_autobccoption[0] == 'Y'
+            || priv->mlfi_conf->cf_autobccoption[0] == 'y'
+        ) {
+            priv->mlfi_bccoption = ENABLE_AUTOBCC;
         }
         else {
-	    priv->mlfi_bccoption = DISABLE_AUTOBCC;
+            priv->mlfi_bccoption = DISABLE_AUTOBCC;
         }
 
     }
-    /* Only for Encription mode --- to here --- */
+
+    if (ismode_harmless) {
+        if (addr != NULL) {
+            char *tmp_ip;
+            if (addr->sa_family == AF_INET) {
+                tmp_ip = malloc(INET_ADDRSTRLEN);
+                if (tmp_ip == NULL) {
+                    log(ERR_MEMORY_ALLOCATE,
+                            "mlfi_connect", "tmp_ip", strerror(errno));
+                    mlfi_cleanup(ctx, MLFIABORT);
+                    return SMFIS_TEMPFAIL;
+                }
+                priv->mlfi_sendercheck_arg.sa.sa_in = *s;
+                priv->mlfi_sendercheck_arg.af = AF_INET;
+
+                if (inet_ntop(AF_INET, &s->sin_addr, tmp_ip, INET_ADDRSTRLEN) == NULL) {
+                    log(ERR_WHITELIST_INVALID_IP, "mlfi_connect", ipv4str);
+                    mlfi_cleanup(ctx, MLFIABORT);
+                    return SMFIS_TEMPFAIL;
+                }
+            } else {
+                tmp_ip = malloc(INET6_ADDRSTRLEN);
+                if (tmp_ip == NULL) {
+                    log(ERR_MEMORY_ALLOCATE,
+                            "mlfi_connect", "tmp_ip6", strerror(errno));
+                    mlfi_cleanup(ctx, MLFIABORT);
+                    return SMFIS_TEMPFAIL;
+                }
+                priv->mlfi_sendercheck_arg.sa.sa_in6 = *s6;
+                priv->mlfi_sendercheck_arg.af = AF_INET6;
+
+                if (inet_ntop(AF_INET6, &s6->sin6_addr, tmp_ip, INET6_ADDRSTRLEN) == NULL) {
+                    log(ERR_WHITELIST_INVALID_IP, "mlfi_connect", ipv4str);
+                    mlfi_cleanup(ctx, MLFIABORT);
+                    return SMFIS_TEMPFAIL;
+                }
+            }
+
+            priv->mlfi_sendercheck_arg.ip = tmp_ip;
+        } else {
+            priv->mlfi_sendercheck_arg.ip = NULL;
+        }
+    }
 
 
     /* create mailinfo structure */
     minfo = malloc(sizeof *minfo);
     if(minfo == NULL) {
-	log(ERR_MEMORY_ALLOCATE, "mlfi_connect", "minfo", strerror(errno));
+        log(ERR_MEMORY_ALLOCATE, "mlfi_connect", "minfo", strerror(errno));
         mlfi_cleanup(ctx, MLFIABORT);
         return SMFIS_TEMPFAIL;
     }
     memset(minfo, 0, sizeof *minfo);
 
-    /* allocate message buffer */
     minfo->ii_mbuf = (char *)malloc(MBSIZE + 1);
     if(minfo->ii_mbuf == NULL) {
-	log(ERR_MEMORY_ALLOCATE, "mlfi_connect", "minfo->ii_mbuf", strerror(errno));
+        log(ERR_MEMORY_ALLOCATE, "mlfi_connect", "minfo->ii_mbuf", strerror(errno));
         mlfi_cleanup(ctx, MLFIABORT);
         return SMFIS_TEMPFAIL;
     }
-    
-    /* Initialize mailinfo structure */
+
     minfo->ii_bufsize = MBSIZE;
 
     priv->mlfi_minfo = minfo;
 
-    /* copy buffer */
     smfi_setpriv(ctx, priv);
 
     return SMFIS_CONTINUE;
@@ -181,8 +237,24 @@ mlfi_connect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *addr)
 sfsistat
 mlfi_helo(SMFICTX *ctx, char *helohost)
 {
+    struct mlfiPriv *priv;
+
+    if ((priv = MLFIPRIV(ctx)) == NULL) {
+        log(ERR_GET_PRIV, "mlfi_helo");
+        mlfi_cleanup(ctx, MLFIABORT);
+        return SMFIS_TEMPFAIL;
+    }
     DEBUGLOG("mlfi_helo start");
 
+    free(priv->mlfi_sendercheck_arg.helo);
+    priv->mlfi_sendercheck_arg.helo = NULL;
+
+    if ((priv->mlfi_sendercheck_arg.helo = strdup(helohost)) == NULL) {
+        log(ERR_GET_PRIV, "mlfi_helo");
+        mlfi_cleanup(ctx, MLFIABORT);
+        return SMFIS_TEMPFAIL;
+    }
+
     return SMFIS_CONTINUE;
 }
 
@@ -199,7 +271,7 @@ mlfi_envfrom(SMFICTX *ctx, char **envfrom)
     char *fqdn;
 
     if ((priv = MLFIPRIV(ctx)) == NULL) {
-	log(ERR_GET_PRIV, "mlfi_envfrom");
+        log(ERR_GET_PRIV, "mlfi_envfrom");
         mlfi_cleanup(ctx, MLFIABORT);
         return SMFIS_TEMPFAIL;
     }
@@ -210,7 +282,7 @@ mlfi_envfrom(SMFICTX *ctx, char **envfrom)
     /* get mail addr */
     fromaddr = get_addrpart((unsigned char *)*envfrom);
     if (fromaddr == NULL) {
-	log(ERR_MAIL_FIND_ADDRESS, "mlfi_envfrom");
+        log(ERR_MAIL_FIND_ADDRESS, "mlfi_envfrom");
         mlfi_cleanup(ctx, MLFIABORT);
         return SMFIS_TEMPFAIL;
     }
@@ -223,6 +295,20 @@ mlfi_envfrom(SMFICTX *ctx, char **envfrom)
         return SMFIS_CONTINUE;
     }
 
+    if (ismode_harmless) {
+        priv->mlfi_savefrom = fromaddr;
+
+        priv->mlfi_sendercheck_arg.envelope_from = strdup(fromaddr);
+        if (priv->mlfi_sendercheck_arg.envelope_from == NULL) {
+            log(ERR_MEMORY_ALLOCATE, "mlfi_envfrom", "enc_from", strerror(errno));
+            mlfi_cleanup(ctx, MLFIABORT);
+            return SMFIS_TEMPFAIL;
+        }
+
+        DEBUGLOG("Sender Accepted");
+        return SMFIS_CONTINUE;
+    }
+
      /* domain null check */
     fqdn = strchr(fromaddr, ATMARK);
     if (fqdn == NULL) {
@@ -239,14 +325,14 @@ mlfi_envfrom(SMFICTX *ctx, char **envfrom)
 
         return SMFIS_TEMPFAIL;
     } else if (ret == NOT_ENC) {
-	if (priv->mlfi_bccoption == DISABLE_AUTOBCC) {
-	    free(fromaddr);
-	    mlfi_cleanup(ctx, MLFIABORT);
+        if (priv->mlfi_bccoption == DISABLE_AUTOBCC) {
+            free(fromaddr);
+            mlfi_cleanup(ctx, MLFIABORT);
 
-	    return SMFIS_ACCEPT;
-	}
-	/* save envription status */
-	priv->mlfi_encstatus = NOT_ENC;
+            return SMFIS_ACCEPT;
+        }
+        /* save envription status */
+        priv->mlfi_encstatus = NOT_ENC;
     }
     /* save from addr temporally */
     priv->mlfi_savefrom = fromaddr;
@@ -264,7 +350,7 @@ mlfi_envrcpt(SMFICTX *ctx, char **rcptto)
     char *rcptaddr = NULL;
     struct mlfiPriv *priv;
     if ((priv = MLFIPRIV(ctx)) == NULL) {
-	log(ERR_GET_PRIV, "mlfi_envrcpt");
+        log(ERR_GET_PRIV, "mlfi_envrcpt");
         mlfi_cleanup(ctx, MLFIABORT);
         return SMFIS_TEMPFAIL;
     }
@@ -274,10 +360,10 @@ mlfi_envrcpt(SMFICTX *ctx, char **rcptto)
     /* get mail addr */
     rcptaddr = get_addrpart_notranslate((unsigned char *)*rcptto);
     if (rcptaddr == NULL) {
-	if (rcptaddr != NULL) {
+        if (rcptaddr != NULL) {
             free(rcptaddr);
-	}
-	log(ERR_MAIL_FIND_ADDRESS, "mlfi_envrcpt");
+        }
+        log(ERR_MAIL_FIND_ADDRESS, "mlfi_envrcpt");
         mlfi_cleanup(ctx, MLFIABORT);
         return SMFIS_TEMPFAIL;
     }
@@ -286,9 +372,9 @@ mlfi_envrcpt(SMFICTX *ctx, char **rcptto)
 
     /* push rcpt addr */
     if (push_rcptlist(&(priv->mlfi_savercpt), rcptaddr) != 0) {
-	mlfi_cleanup(ctx, MLFIABORT);
-	free(rcptaddr);
-	return SMFIS_TEMPFAIL;
+        mlfi_cleanup(ctx, MLFIABORT);
+        free(rcptaddr);
+        return SMFIS_TEMPFAIL;
     }
     free(rcptaddr);
 
@@ -311,13 +397,24 @@ mlfi_header(SMFICTX *ctx, char *headerf, char *headerv)
     struct rcptinfo *passlist = NULL, *rdmpasslist = NULL;
 
     if ((priv = MLFIPRIV(ctx)) == NULL) {
-	log(ERR_GET_PRIV, "mlfi_header");
+        log(ERR_GET_PRIV, "mlfi_header");
         mlfi_cleanup(ctx, MLFIABORT);
         return SMFIS_TEMPFAIL;
     }
 
     DEBUGLOG("mlfi_header start");
 
+    if (strcasecmp(headerf, "Message-Id") == 0) {
+        free(priv->mlfi_sendercheck_arg.message_id);
+        priv->mlfi_sendercheck_arg.message_id = NULL;
+        priv->mlfi_sendercheck_arg.message_id = strdup(headerv);
+        if (priv->mlfi_sendercheck_arg.message_id == NULL) {
+            log(ERR_MEMORY_ALLOCATE, "mlfi_header", "message-id", strerror(errno));
+            mlfi_cleanup(ctx, MLFIABORT);
+            return SMFIS_TEMPFAIL;
+        }
+    }
+
     // Cases for priv->mlfi_encstatus == NOT_ENC:
     //  o Not-Encryption due to the fromDB.
     //  o Not-Encryption as a result of the following rcptcheck.
@@ -328,8 +425,8 @@ mlfi_header(SMFICTX *ctx, char *headerf, char *headerv)
     /* START ADD: 20150323 */
     /* check loop encryption */
     if ((strcmp(priv->mlfi_conf->cf_loopcheck, LOOP_CHECK) == 0 ) &&
-	(strcmp(headerf, XHEADER) == 0) && 
-	(strcmp(headerv, XHEADER_YES) == 0)) {
+        (strcmp(headerf, XHEADER) == 0) &&
+        (strcmp(headerv, XHEADER_YES) == 0)) {
         mlfi_cleanup(ctx, MLFIABORT);
         return SMFIS_ACCEPT;
     }
@@ -338,7 +435,7 @@ mlfi_header(SMFICTX *ctx, char *headerf, char *headerv)
     /* RcptTo Check */
     if (priv->mlfi_rcptcheck == BEFOR_RCPTCHECK) {
 
-        if (!ismode_delete) {
+        if (ismode_enc) {
             /* Encryption Mode */
             if (priv->mlfi_bccoption == ENABLE_AUTOBCC) {
                 ret = add_bccaddr(priv->mlfi_conf, priv->mlfi_savefrom,
@@ -349,11 +446,12 @@ mlfi_header(SMFICTX *ctx, char *headerf, char *headerv)
                 }
             }
 
-            ret = search_rcptaddr(priv->mlfi_conf, priv->mlfi_savefrom, 
+            ret = search_rcptaddr(priv->mlfi_conf, priv->mlfi_savefrom,
                     priv->mlfi_savercpt, &passlist, &rdmpasslist);
         } else {
             /* Delete Mode */
-            ret = search_rcptaddr_delmode(priv->mlfi_conf, 
+            /* harmless Mode */
+            ret = search_rcptaddr_delmode(priv->mlfi_conf,
                     priv->mlfi_savercpt, &passlist, &rdmpasslist);
             // Delete recipients are added to passlist.
         }
@@ -425,17 +523,39 @@ sfsistat
 mlfi_eoh(SMFICTX *ctx)
 {
     struct mlfiPriv *priv;
+    int ret;
 
     DEBUGLOG("mlfi_eoh start");
 
     if ((priv = MLFIPRIV(ctx)) == NULL) {
-	log(ERR_GET_PRIV, "mlfi_eoh");
+        log(ERR_GET_PRIV, "mlfi_eoh");
         mlfi_cleanup(ctx, MLFIABORT);
         return SMFIS_TEMPFAIL;
     }
 
     if (priv->mlfi_encstatus == NOT_ENC) {
-	return SMFIS_CONTINUE;
+        return SMFIS_CONTINUE;
+    }
+
+    if (ismode_harmless) {
+        ret = check_sender(priv);
+        switch (ret) {
+            case SENDER_CHECK_OK:
+                return SMFIS_ACCEPT;
+            case SENDER_CHECK_NG:
+                log(
+                    LOG_HARMLESS
+                    , priv->mlfi_sendercheck_arg.ip
+                    , priv->mlfi_sendercheck_arg.message_id
+                    , priv->mlfi_sendercheck_arg.envelope_from
+                    , priv->mlfi_sendercheck_arg.rcpt_to
+                );
+                break;		// 無害化処理を行う
+            case SENDER_CHECK_ERR:
+            default:
+                mlfi_cleanup(ctx, MLFIABORT);
+                return SMFIS_TEMPFAIL;
+        }
     }
 
     /* write new line between header and body */
@@ -458,7 +578,7 @@ mlfi_body(SMFICTX *ctx, u_char *bodyp, size_t bodylen)
 {
     struct mlfiPriv *priv;
     if ((priv = MLFIPRIV(ctx)) == NULL) {
-	log(ERR_GET_PRIV, "mlfi_body");
+        log(ERR_GET_PRIV, "mlfi_body");
         mlfi_cleanup(ctx, MLFIABORT);
         return SMFIS_TEMPFAIL;
     }
@@ -466,7 +586,7 @@ mlfi_body(SMFICTX *ctx, u_char *bodyp, size_t bodylen)
     DEBUGLOG("mlfi_body start");
 
     if (priv->mlfi_encstatus == NOT_ENC) {
-	return SMFIS_CONTINUE;
+        return SMFIS_CONTINUE;
     }
 
     /* write body */
@@ -490,7 +610,7 @@ mlfi_eom(SMFICTX *ctx)
     int ret, i;
     struct mlfiPriv *priv;
     if ((priv = MLFIPRIV(ctx)) == NULL) {
-	log(ERR_GET_PRIV, "mlfi_eom");
+        log(ERR_GET_PRIV, "mlfi_eom");
         mlfi_cleanup(ctx, MLFIABORT);
         return SMFIS_TEMPFAIL;
     }
@@ -498,29 +618,31 @@ mlfi_eom(SMFICTX *ctx)
     DEBUGLOG("mlfi_eom start");
 
     if (priv->mlfi_savebcc != NULL) {
-	for (i = 0; (priv->mlfi_savebcc + i)->rcpt_addr != NULL; i++) {
-	    if (smfi_addrcpt(ctx, (priv->mlfi_savebcc + i)->rcpt_addr) != MI_SUCCESS) {
-		log(ERR_AUTOBCC, "mlfi_eom", "smfi_addrcpt() failed.");
-		mlfi_cleanup(ctx, MLFIABORT);
-		return SMFIS_TEMPFAIL;
-	    }
-	}
+        for (i = 0; (priv->mlfi_savebcc + i)->rcpt_addr != NULL; i++) {
+            if (smfi_addrcpt(ctx, (priv->mlfi_savebcc + i)->rcpt_addr) != MI_SUCCESS) {
+        	log(ERR_AUTOBCC, "mlfi_eom", "smfi_addrcpt() failed.");
+        	mlfi_cleanup(ctx, MLFIABORT);
+        	return SMFIS_TEMPFAIL;
+            }
+        }
     }
 
     if (priv->mlfi_encstatus == NOT_ENC) {
-	mlfi_cleanup(ctx, MLFIABORT);
-	return SMFIS_ACCEPT;
+        mlfi_cleanup(ctx, MLFIABORT);
+        return SMFIS_ACCEPT;
     }
 
     if (ismode_delete) {
         /* Delete Attachments */
-        ret = delete_attachments_mail(ctx, priv->mlfi_minfo, 
+        ret = delete_attachments_mail(ctx, priv->mlfi_minfo,
                 priv->mlfi_conf, priv->mlfi_savefrom,
                 priv->mlfi_rdmpasslist);
+    } else if (ismode_harmless) {
+        ret = harmless(ctx, priv);
     } else {
         /* Encryption Process */
-        ret = zip_convert_mail(ctx, priv->mlfi_minfo, priv->mlfi_conf, 
-                priv->mlfi_savefrom, priv->mlfi_passlist, 
+        ret = zip_convert_mail(ctx, priv->mlfi_minfo, priv->mlfi_conf,
+                priv->mlfi_savefrom, priv->mlfi_passlist,
                 priv->mlfi_rdmpasslist);
     }
 
@@ -537,7 +659,7 @@ mlfi_eom(SMFICTX *ctx)
                           MESSAGE_INVALID_FILENAME) != MI_SUCCESS) {
             return SMFIS_TEMPFAIL;
         }
-	return SMFIS_REJECT;
+        return SMFIS_REJECT;
     }
 
     return SMFIS_CONTINUE;
@@ -587,46 +709,63 @@ mlfi_cleanup(SMFICTX *ctx, int flag)
     /* priv release */
     if (priv->mlfi_savefrom != NULL) {
         free(priv->mlfi_savefrom);
-	priv->mlfi_savefrom = NULL;
+        priv->mlfi_savefrom = NULL;
     }
 
     if (priv->mlfi_savercpt != NULL) {
         free_rcptlist(priv->mlfi_savercpt);
-	priv->mlfi_savercpt = NULL;
+        priv->mlfi_savercpt = NULL;
     }
 
     if (priv->mlfi_passlist != NULL) {
         free_rcptinfo(priv->mlfi_passlist);
-	priv->mlfi_passlist = NULL;
+        priv->mlfi_passlist = NULL;
     }
 
     if (priv->mlfi_rdmpasslist != NULL) {
         free_rcptinfo(priv->mlfi_rdmpasslist);
-	priv->mlfi_rdmpasslist = NULL;
+        priv->mlfi_rdmpasslist = NULL;
     }
 
     if (priv->mlfi_savebcc != NULL) {
-	free_rcptlist(priv->mlfi_savebcc);
-	priv->mlfi_savebcc = NULL;
+        free_rcptlist(priv->mlfi_savebcc);
+        priv->mlfi_savebcc = NULL;
     }
 
+    free(priv->mlfi_sendercheck_arg.ip);
+    priv->mlfi_sendercheck_arg.ip = NULL;
+
+    free(priv->mlfi_sendercheck_arg.helo);
+    priv->mlfi_sendercheck_arg.helo = NULL;
+
+    free(priv->mlfi_sendercheck_arg.envelope_from);
+    priv->mlfi_sendercheck_arg.envelope_from = NULL;
+
+    free(priv->mlfi_sendercheck_arg.message_id);
+    priv->mlfi_sendercheck_arg.message_id = NULL;
+
+    free(priv->mlfi_sendercheck_arg.rcpt_to);
+    priv->mlfi_sendercheck_arg.rcpt_to = NULL;
+
+    priv->mlfi_sendercheck_arg.af = 0;
+
     if (flag == MLFICLOSE) {
         /* config relase */
         if (priv->mlfi_conf != NULL) {
             config_release(priv->mlfi_conf);
-	    priv->mlfi_conf = NULL;
+            priv->mlfi_conf = NULL;
         }
 
-	if (priv->mlfi_minfo != NULL) {
-	    mailsave_clean(priv->mlfi_minfo);
-	    free(priv->mlfi_minfo);
-	    priv->mlfi_minfo = NULL;
-	}
+        if (priv->mlfi_minfo != NULL) {
+            mailsave_clean(priv->mlfi_minfo);
+            free(priv->mlfi_minfo);
+            priv->mlfi_minfo = NULL;
+        }
 
-	free(priv);
+        free(priv);
 
-	/* set NULL */
-	smfi_setpriv(ctx, NULL);
+        /* set NULL */
+        smfi_setpriv(ctx, NULL);
     }
 
     return rstat;
@@ -641,7 +780,7 @@ call_command(void *arg)
     tcl = (struct thread_control *)arg;
 
     if (accept_command(tcl->configname , tcl->addr, tcl->port) != 0) {
-	log(ERR_RELOAD_SERVER, "call_command");
+        log(ERR_RELOAD_SERVER, "call_command");
     }
 
     pthread_exit(&ret);
@@ -649,19 +788,25 @@ call_command(void *arg)
 
 /*
  * usage
- * 
+ *
  */
 void
 usage(char *arg)
 {
-    fprintf(stderr, "usage: %s [-d] [-t timeout] [config file]\n", arg);
+    fprintf(stderr, "usage: %s [-v] [-d|-h] [-t timeout] [config file]\n", arg);
+}
+
+void
+version()
+{
+    printf("SaMMA (%s) - SAfety Mail gateway with Milter Api\n\tDesignet.inc\n", SAMMA_VERSION);
 }
 
 int
 main(int argc, char *argv[])
 {
 
-    char  *args = "dt:";
+    char  *args = "dhvt:";
     char  *confname;
     char  *tmp;
     char   c;
@@ -675,7 +820,7 @@ main(int argc, char *argv[])
     srand48(time(NULL));
 
     /* ignore sigpipe signal */
-    signal(SIGPIPE, SIG_IGN); 
+    signal(SIGPIPE, SIG_IGN);
 
     /* set error output to stderr */
     init_log();
@@ -695,7 +840,7 @@ main(int argc, char *argv[])
             tmp_val = strtoul(optarg, &tmp, 10);
             if((*tmp != '\0') ||
                ((tmp_val == ULONG_MAX) && (errno == ERANGE)) ||
-	       (tmp_val > INT_MAX)) {
+               (tmp_val > INT_MAX)) {
                 usage(argv[0]);
                 exit(1);
             }
@@ -704,10 +849,19 @@ main(int argc, char *argv[])
             break;
         case 'd':
             /* Delete Mode */
+            ismode_enc = 0;
             ismode_delete = 1;
-
             break;
 
+        case 'h':
+            ismode_enc = 0;
+            ismode_harmless = 1;
+            break;
+
+        case 'v':
+            version();
+            exit(1);
+
         /* error */
         default:
             usage(argv[0]);
@@ -717,18 +871,23 @@ main(int argc, char *argv[])
 
     /* check arg */
     if (argc > MAX_ARGS) {
-	usage(argv[0]);
-	exit(1);
+        usage(argv[0]);
+        exit(1);
     } else if(optind + 1 == argc) {
 
-	/* set config name */
-	confname = argv[optind];
+        /* set config name */
+        confname = argv[optind];
     } else if (optind == argc) {
-	/* set default config name */
-	confname = DEFAULT_CONFFILE;
+        /* set default config name */
+        confname = DEFAULT_CONFFILE;
     } else {
-	usage(argv[0]);
-	exit(1);
+        usage(argv[0]);
+        exit(1);
+    }
+
+    if (ismode_delete * ismode_harmless) {
+        usage(argv[0]);
+        exit(1);
     }
 
     /* Select config entries. */
@@ -736,8 +895,8 @@ main(int argc, char *argv[])
 
     /* read config file */
     if (reload_config(confname) != 0) {
-	fprintf(stderr, STDERR_CONFIG_READ, confname);
-	fprintf(stderr, "\n");
+        fprintf(stderr, STDERR_CONFIG_READ, confname);
+        fprintf(stderr, "\n");
         exit(2);
     }
 
@@ -748,9 +907,9 @@ main(int argc, char *argv[])
     tcl.port = cfg->cf_commandport;
     tcl.addr = strdup(cfg->cf_listenip);
     if (tcl.addr == NULL) {
-	fprintf(stderr, STDERR_MEMORY_ALLOCATE, "tcl.addr", strerror(errno));
-	fprintf(stderr, "\n");
-	exit(2);
+        fprintf(stderr, STDERR_MEMORY_ALLOCATE, "tcl.addr", strerror(errno));
+        fprintf(stderr, "\n");
+        exit(2);
     }
 
     /* initialize GMime */
@@ -768,16 +927,16 @@ main(int argc, char *argv[])
     /* set socket */
     if(smfi_setconn(oconn) == MI_FAILURE) {
         fprintf(stderr, ERR_MILTER_SET_SOCKET, strerror(errno));
-	fprintf(stderr, "\n");
-	g_mime_shutdown();
+        fprintf(stderr, "\n");
+        g_mime_shutdown();
         exit(3);
     }
 
     /* set time out */
     if(smfi_settimeout(timeout) == MI_FAILURE) {
         fprintf(stderr, ERR_MILTER_SET_TIMEOUT, strerror(errno));
-	fprintf(stderr, "\n");
-	g_mime_shutdown();
+        fprintf(stderr, "\n");
+        g_mime_shutdown();
         exit(3);
     }
 
@@ -785,16 +944,16 @@ main(int argc, char *argv[])
     smfilter.xxfi_flags |= SMFIF_ADDRCPT;
     if(smfi_register(smfilter) == MI_FAILURE) {
         fprintf(stderr, ERR_MILTER_REGISTER, strerror(errno));
-	fprintf(stderr, "\n");
-	g_mime_shutdown();
+        fprintf(stderr, "\n");
+        g_mime_shutdown();
         exit(4);
     }
 
     /* run main */
     if(smfi_main() == MI_FAILURE) {
         fprintf(stderr, ERR_MILTER_START, strerror(errno));
-	fprintf(stderr, "\n");
-	g_mime_shutdown();
+        fprintf(stderr, "\n");
+        g_mime_shutdown();
         exit(5);
     }
 
@@ -805,7 +964,7 @@ main(int argc, char *argv[])
 
 /* for debug  */
 #ifdef DEBUG
-void 
+void
 listvardump(struct rcptinfo *passlist)
 {
     int i;
