@@ -909,7 +909,7 @@ encode_tmplheader(char **mail)
 
         // other header field
         } else {
-            ret = alloc_and_cat(&header, 1, one_line);
+            ret = alloc_and_cat(&header, 2, one_line, "\n");
         }
         free(one_line);
 
